/*
 * BMA180.c
 *
 * Created: 06.01.2011 12:42
 *  Author: martin
 */ 

#ifndef F_CPU
	#define F_CPU 8000000UL 	       
#endif

#include <avr/interrupt.h>
#include <avr/io.h>
#include "BMA020.h"
#include <util/delay.h>

//-----------------------------------------------------------------------------

int init_BMA020(uint8_t range, uint8_t bw)
{
	char temp,temp1;
	//if connected correctly, ID register schould be 3
	if(read(Chip_ID)!=2)
		return -1;
	//------------------------------------------------
	
	//Set Bandwidth
	temp=read(Bw_Range);
	temp1=bw;
	temp|=temp1;			//gewnschte Bandbreite BW wird in temp im High-Byte gesetzt
	write(Bw_Range,temp);		//keep tcs<3:0> in BWCTS, but write new BW
	//------------------------------------------------
	//Set Range
	temp=read(Bw_Range);
	temp1=range;
	temp1=(temp1<<RANGESHIFT);
	temp&=(~RANGEMASK);
	temp|=temp1;
	write(Bw_Range,temp);		//Write new range data, keep other bits the same
	//------------------------------------------------
	
	return 0;
}


void write(uint8_t address, char data)
{
	//write any data byte to any single address
	//adds a "0" to the MSB of the address byte (Write Mode)
	
	address&=0x7F;
	
	cbi(PORTB,CSB);
	txdata(address);
	_delay_ms(1);
	txdata(data);
	sbi(PORTB,CSB);
}

char read(uint8_t address)
{
	//returns the contents of any One-Byte-Register from any address
	//sets the MSB for every address byte (READ Mode) 

	char byte;
	
	address|=0x80;
	
	cbi(PORTB,CSB);
	txdata(address);
	byte=rxdata();
	sbi(PORTB,CSB);
	
	return byte;
}

char rxdata(void)
{
	SPDR=0x55;
	while((SPSR&0x80)==0x00);
	
	return SPDR;
}

void txdata(char data)
{
	SPDR = data;
	while((SPSR&0x80)==0x00);
}

void init_SPI(void)							
{
	sbi(SPCR,MSTR);		//make SPI Master
	sbi(SPCR,CPOL);
	sbi(SPCR,CPHA);		//SCL idle high, sample data on rising edge
	cbi(SPCR,SPR1); cbi(SPCR,SPR0); cbi(SPSR,SPI2X); //Fosc/4 is SPI frequency 4,6MHz@18,432MHz_MCU
	sbi(SPCR,SPE);		//enable SPI
}



   