/** \mainpage Siemens Handy
* \date Feb 2010
* \author Lars Neubert, neubert.lars@googlemail.com
* 
* \brief Projekt zur Kommunikation mit Mobiltelefonen
*
* Das Projekt dient zur Kommunikation mit Mobiltelefonen.
*<br> Das beinhaltet:
* - die Steuerung des Mobiltelefons mittels AT-Kommandos
* - die Mglichkeit zum Versenden von Textnachrichten im PDU Format
* - die Mglichkeit des Empfangs von Textnachrichten im PDU Format und anschlieender Dekodierung
* - das Auslesen der Datums- und Zeitangaben des angeschlossenen Mobiltelefons
*
* Die eigentliche Kommunikation mit dem Mobiltelefon erfolgt dabei ber die serielle UART Schnittstelle 
*<br> des Mikrocontrollers. Zur Vereinfachung wurde dabei die Standardein und -ausgabe auf die blichen
*<br> C Funktionen wie printf, gets, scanf, etc. umgeleitet. 
*<br> Neben der Kommunikation mit dem angeschlossenen Mobiltelefon, besteht zudem die Mglichkeit einen
*<br> digitalen I2C Sensor anzusteuern. Beispielhaft ist durch die Verwendung des TWI Interfaces des 
*<br> Mikrocontrollers eine Buskommunikation mit dem Temperatursensor DS75LV (und quivalente) in diesem  
*<br> Projekt implementiert.
*/

/**
* \file main.c
* @brief Hauptdatei des Projektes
* \author Lars Neubert
* \date Feb 2010
*/

#ifndef F_CPU
#define F_CPU 8000000UL
#endif

#include "main.h"
#include "BMA020.h"

/**
* globales Flag, welches Signalisiert, dass Timer1 ausgelst wurde
*/
volatile uint8_t timer_flag;


int main(void) {

	//uint8_t state=0;
	//char s[20]={'0'};

	//unsigned char year=0;
	//unsigned char month=0;
	//unsigned char day=0;
	//unsigned char hour=0;
	//unsigned char minute=0;
	//unsigned char second=0; 

	DDRB = 0b10111111;
	PORTB |= (1<<PB6); 
	DDRC = 0b11011111;
	PORTC |= (1<<PC5); 
	
	PORTC|=(1<<PC3);	//Betriebs-LED

	// 16 bit timer:  
    TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10);			// CTC + Prescaler 1024 --> CPU/1024
	//TCCR1B = (1<<WGM12)|(1<<CS11)|(1<<CS10);			// CTC + Prescaler 256 --> CPU/256
	// bei 16MHz und prescaler 1024 --> 1s --> OCR1A  = 15624
	// bei 1MHz und prescaler 64 --> 1s	--> OCR1A  = 15624
    OCR1A  = 15624;							//vorladen, f_comp=fosc/(prescaler*(OCRnX+1)); OCRnX=fosc/(f_comp*prescaler)-1
    #ifdef __AVR_ATmegaM32__
		TIMSK |= (1<<OCIE1A);
	#endif
	#if __AVR_ATmegaM168A	
		TIMSK1 |= (1<<OCIE1A);
		        			//Output Compare Match A aktivieren
	#endif
    
	// 8 bit timer: 
	#ifdef __AVR_ATmegaM32__
	{
		TCCR2 = (1<<WGM21)|(1<<CS22)|(1<<CS21)	//TCCR2 = (1<<WGM21)|(1<<CS21);			// CTC + Prescaler 8 --> CPU/8
												// bei 16MHz und prescaler 256 --> 2ms --> OCR2  = 124;
												// bei 1MHz und prescaler 8 --> 2ms --> OCR2  = 249;
    	OCR2 = 124;
		TIMSK |= (1<<OCIE2);
	}		
	#endif
	#ifdef  __AVR_ATmegaM168A__ 
	{
		TCCR2A |= (1<<WGM21);
		TCCR2B = (1<<CS22)|(1<<CS21);			// CTC + Prescaler 256 --> CPU/256
		OCR2A  = 124;							//vorladen, f_comp=fosc/(prescaler*(OCRnX+1)); OCRnX=fosc/(f_comp*prescaler)-1
		TIMSK2 |= (1<<OCIE2A);        			//Output Compare Match aktivieren
	}	
	#endif
	
	

	
	timer_flag = 0;

	uart_clear();


	sei();
	init_SPI();
    sbi(PORTB,CSB);
   
    init_BMA020(_2g_,_25Hz_);	
	
	sms_init();
	//sms_time (&year, &month, &day, &hour, &minute, &second);
	
	//sprintf(s,"%02d%02d%02d-%02d:%02d:%02d",day, month, year, hour, minute, second);
	
	sms_display_text("Handy: bereit");		//sendet Nachricht aufs Handydisplay
	_delay_ms(2000);					//Verzgerungszeit, damit Nachricht lnger auf Display zu sehen ist
	printf("%c",13);					// <\r> Carriage Return, damit nchster Befehl ausgefhrt werden kann
	_delay_ms(500);						//Wartezeit nach jedem Befehl
	
	
	write(CTRLREG,reset_INT);
	
	//Befehle fr BMA020
	write(enableREG,enable_ADV_INT);
	write(INT_Mode,any_motion_INT);
	write(any_motion_dur,dur_7); 
	write(any_motion_thres,0x08);	//Any_motion_thres: 0x08 -> 08*0,015625=0,125g
									//                  0x10 -> 16*0,015625=0,25g 
									//					0x17 -> 23*0,015625=0,36g
									//					0x20 -> 32*0,015625=0,5g
	
	while (1) {
		
		
		if(PINC & (1<<INT))			//Bewegungsdetektion mit Beschleunigungssensor BMA020 -> Any_Motion_Interrupt
		{
			PORTC |= (1<<PC4);		//Kontrolllampe
			sms_send(SMS_TELNR,"Test: ERFOLGREICH :-)");
			_delay_ms(500); 
			write(CTRLREG,reset_INT);
		    _delay_ms(100);
		    PORTC &= ~(1<<PC4);
		}		
		
		//state=sms_state_machine();
		//if (state==SMS_RETRIEVE_TELNR) {
			
		//}		

		// Timer1 ist abgelaufen		
		if ( timer_flag==1 ) 
		{	
			timer_flag=0;
		}
	}
return 0;
}



// bei 16MHz und prescaler 1024 --> 1s
ISR (TIMER1_COMPA_vect)
{
	if ( timer_flag==0) {timer_flag=1;}
	OCR1A  = 15624;				//f_comp=fosc/(prescaler*(OCRnX+1))
}

// bei 16MHz und prescaler 256 --> 2ms
ISR (TIMER2_COMP_vect)
{
	#ifdef __AVR_ATmegaM32__
		OCR2 = 124;
	#endif
	#ifdef __AVR_ATmegaM168A__
		OCR2A  = 124;				//f_comp=fosc/(prescaler*(OCRnX+1))
	#endif
	
	uart_timeout++;
	if (uart_timeout>250) {
		uart_timeout=0;
	}
}



