/**
* \file sms.c
* @brief ermglicht das Senden einer Textnachricht ber ein Mobiltelefon
* \author Lars Neubert
* \date March 2010
*/


#include "sms.h"
#include "uart.h"


SMS_DECODE_DATA_T sms_dec_data;
SMS_TELNR_LIST_T sms_telnr_list[SMS_TELNR_LIST_MAX];
char sms_last_caller[SMS_TELNR_MAX_LEN+1]={""};
uint8_t anz_klingeln=0;

/**
* konkrete Zuordnung von Befehlsstrings zu einer Funktion vom Typ "uint8_t <fname> (uint8_t dat)"
*/
const SMS_CMD_LIST_T sms_cmd_list[] = {
	{"set",sms_cmdset}
};



/**
* Wandelt ein Zeichen in eine Hexzahl um
* @param data Zeichen, welches umgewandelt werden soll
* @return Hexzahl equivalent von 'data'
*/
uint8_t str2int(char data){
  data -= '0';
  if (data>9) data-= 7;
  return data;
}


/**
* Initialisiert die serielle Schnittstelle und ffnet einen Kanal fr Standard Ein-/Ausgabe (printf, gets, scanf,...)
*/
void sms_init(void){
	// Initialisierung der seriellen Schnittstelle Asynchron
	uart_init();
	// ffnet Kanal fr printf
	//stdout = &mystdout;
	fdevopen (uart_sendc, uart_receive);

	UCSRB &= ~(1 << RXCIE);		// UART Receive ISR ausstellen

	//printf("ATE0\r\n");		// Echo off, reduce traffic
	_delay_ms(200);
	printf("AT+CPMS=\"ME\",\"MT\",\"MT\"\r\n");		// Speicher von gelesenen, geschriebenen und empfangenen SMS auf Telefonspeicher stellen
	_delay_ms(200);
	printf("AT+CNMI=1,1,0,0,1\r\n");				// new message indication --> "+CMTI:<mem>,<index>"
	_delay_ms(200);

	// Auslesen der ersten 5 Telefonnummern der SIM-Karte
	char telnr[SMS_TELNR_MAX_LEN+1];
	uint8_t i=0;
	int dummy;
	printf("AT+CPBS=\"SM\"\r\n");					// Telefonbuch auf SIM Karte umstellen
	_delay_ms(200);
	uart_clear();
	for (i=0;i<SMS_TELNR_LIST_MAX;i++) {
		uart_clear();
		printf("AT+CPBR=%i\r\n",i+1);
		gets(telnr);
		scanf("+CPBR: %i,\"%[^\"]",&dummy,telnr);
		_delay_ms(400);								// Rest der Antwort abwarten und verwerfen
		if (strlen(telnr)>3) {
			strcpy(sms_telnr_list[i].telnr,telnr);	// ausgelesene Telefonnummer in Struktur speichern	
		}
		else { strcpy(sms_telnr_list[i].telnr, "noNR");}
	}
	uart_clear();
	if (UART_ISR!=0) {UCSRB |= (1 << RXCIE);}		// UART Receive ISR wieder einstellen
}


/**
* Liet Datum und Zeit aus dem Mobiltelefon aus
* @param year Pointer auf Speicherstelle fr Jahreszahl
* @param month Pointer auf Speicherstelle fr Monat
* @param day Pointer auf Speicherstelle fr Tag
* @param hour Pointer auf Speicherstelle fr Stunde
* @param minute Pointer auf Speicherstelle fr Minute
* @param second Pointer auf Speicherstelle fr Sekunde
*/
void sms_time(	unsigned char *year,unsigned char *month,
			unsigned char *day,unsigned char *hour,
			unsigned char *minute,unsigned char *second){
	char tmp[30]={'\0'};
	unsigned int time_var[6]={0,0,0,0,0,0};
	UCSRB &= ~(1 << RXCIE);		// UART Receive ISR ausstellen
	uart_clear();
	printf("at+cclk?\r\n");
	gets(tmp);	// fhrendes \r\n verwerfen
	scanf("+CCLK: \"%2d/%2d/%2d,%2d:%2d:%2d+  \"\r\n",&time_var[0],&time_var[1],&time_var[2],&time_var[3],&time_var[4],&time_var[5]);
	_delay_ms(200);		// auf Antwort von +CCLK warten
	*year = time_var[0];
	*month = time_var[1];
	*day = time_var[2];
	*hour= time_var[3];
	*minute = time_var[4];
	*second = time_var[5];
	uart_clear();	// Besttigung "\r\nOK\r\n von CCLK lschen
	if (UART_ISR!=0) {UCSRB |= (1 << RXCIE);}		// UART Receive ISR wieder einstellen
};


/**
* Sendet eine SMS mit dem Inhalt "nachricht" an "zielnr"
* @param zielnr Pointer auf Zielnummer der SMS
* @param nachricht Pointer auf Nachricht der SMS an Zielnummer
*/
void sms_send(char* zielnr, char* nachricht)
{
	//char tmp1[20]="";
	uint8_t cnt_shiftzero=0;
	uint8_t cnt_nr=0;					// Zhler fr Zielnummer, nach Umwandlung = Lnge der Zielnummer
	uint8_t cnt_txt=0;					// Zhler fr Nachricht, nach Umwandlung = Lnge der Nachricht
	uint8_t shift=0;
	uint8_t i=0;
	char nachricht_neu[SMS_MESSAGE_MAX_LEN+1]={"\0"};
	char zielnr_neu[SMS_TELNR_MAX_LEN+1]={" "};

	UCSRB &= ~(1 << RXCIE);		// UART Receive ISR ausstellen

	// Nachricht umwandeln in PDU-Format
	while (nachricht[cnt_txt]!=0)
	{
		if (shift==7) {
			shift=0;
			cnt_txt++;
			cnt_shiftzero++;
		}
		else {								
			nachricht_neu[cnt_txt-cnt_shiftzero] = (((nachricht[cnt_txt]&0x7F)>>shift) | ((nachricht[cnt_txt+1])<<(7-shift)));
			shift++;
			cnt_txt++;
		}
	}
	nachricht_neu[cnt_txt-cnt_shiftzero]=nachricht[cnt_txt];	// '0' anfgen , Endestring
	
	// Zielnummer neu zusammensetzen
	// Bsp. 01 62 64 94 30 0 --> 10 26 46 49 03 F0
	// wenn 2 Bytes nicht voll ausgenutzt werden, 'F' einfgen
	while (zielnr[cnt_nr]!=0) {
		if (zielnr[cnt_nr+1]==0) {
			zielnr_neu[cnt_nr]='F';	
			zielnr_neu[cnt_nr+1]=zielnr[cnt_nr];
			cnt_nr++;
			cnt_nr++;
			zielnr_neu[cnt_nr]=zielnr[cnt_nr-1];	// '0' anfgen , Endestring
			break;
		}
		else {
			zielnr_neu[cnt_nr]=zielnr[cnt_nr+1];
			zielnr_neu[cnt_nr+1]=zielnr[cnt_nr];
			cnt_nr=cnt_nr+2;
		}
	}

	//Anzahl der Zeichen im PDU-String/2-->Hex Paare -2 (ersten 2 Hex Zeichen weglassen)
	//-->'Anzahl der Nachrichtenzeichen' + 'Anzahl der Nummern /2' (/2 da nur Hex Paare gezhlt werden) + '7' (zustzliche Header)
	if (zielnr_neu[cnt_nr-2]=='F') {
		printf("AT+CMGS=%i",cnt_txt-cnt_shiftzero+(cnt_nr/2)+7);
		cnt_nr--;		//Lnge der Zielnummer -1, da 'F' angefgt
	}
	else {
		printf("AT+CMGS=%i",cnt_txt-cnt_shiftzero+(cnt_nr/2)+7);
	}
    //gets(tmp1);
	printf("%c",13);
	//while(uart_receive(NULL)!=' ');		// auf antwort von Handy warten '> '
	_delay_ms(500);
	printf("000100%02X81",cnt_nr);
	printf(zielnr_neu);
	printf("0000%02X",cnt_txt);
	for (i = 0;i<(cnt_txt-cnt_shiftzero);i++){
		printf("%02X", nachricht_neu[i]);
	};
	printf("%c",26);
	if (UART_ISR!=0) {UCSRB |= (1 << RXCIE);}		// UART Receive ISR wieder einstellen)
}


/**
* Decodiert eine SMS im PDU Format
* @param pdu Pointer aud den komplette PDU-String (angefangen mit Lnge des Headers, SMSC Nr, etc)
* @param data Pointer auf Struktur vom Typ SMS_DECODE_DATA_TYPE (in sms.h definiert) 
* <br> zum speichern der decodierten Daten (Nachricht in Klartext, Telefonnummer, Datum, Zeit)
*/
void sms_decode (char* pdu, SMS_DECODE_DATA_T* data)
{
	if (strlen(pdu)>28) 
	{
		uint8_t cnt_txt=0;					// Zhler fr Nachricht, nach Umwandlung = Lnge der Nachricht ohne \0
		uint8_t shift=7;
		uint8_t i=0;
		uint8_t j=0;
		uint8_t hex=0;
		uint8_t tmp=0;
		uint8_t mask=0x7F;

		tmp = str2int(pdu[0])*16 + str2int(pdu[1]); // lnge des Header
		i = 1 ;	// Header berspringen (pdu[0] und pdu [1])
		// Nummerntyp und SMSC Nummer berspringen
		for (tmp=tmp*2+1; tmp>0; tmp--){
			i++;
		}
		// SMS deliverer berspringen
		i++;
		i++;
	
		// Lnge der Absendernummer
		tmp = str2int(pdu[i])*16 + str2int(pdu[i+1]);
		i++; i++;
		// Nummerntyp berspringen
		i++; i++;

		// Nummer auslesen und speichern
		while (tmp>0){
			hex = pdu[i];
			i++;
			data->telnr[j] = pdu[i];
			j++;
			data->telnr[j] = hex;
			j++; i++;
			if (j>=tmp) {
				data->telnr[tmp]='\0';
				break; 
			}
		}
	
		// Protocol Identifier berspringen
		i++; i++;

		// Data Encoding Scheme berspringen
		i++; i++;

		// timestamp

		data->date.year = str2int(pdu[i+1])*10 + str2int(pdu[i]);
		i++;i++;
		data->date.month = str2int(pdu[i+1])*10 + str2int(pdu[i]);
		i++;i++;
		data->date.day = str2int(pdu[i+1])*10 + str2int(pdu[i]);
		i++;i++;
		data->date.hour = str2int(pdu[i+1])*10 + str2int(pdu[i]);
		i++;i++;
		data->date.minute = str2int(pdu[i+1])*10 + str2int(pdu[i]);
		i++;i++;
		data->date.second = str2int(pdu[i+1])*10 + str2int(pdu[i]);
		i++;i++;
		// Zeitzone/GMT berspringen
		i++;i++;

		// Lnge des Textes berspringen
		i++;i++;

		// Nachricht umwandeln in PDU-Format
		tmp=0;
		while (pdu[i+1]!=0)
		{
			hex = str2int(pdu[i])*16 + str2int(pdu[i+1]);
			data->nachricht[cnt_txt] = ((hex&mask) << (7-shift)) | tmp;
			tmp=hex>>shift;	
			shift--;
			cnt_txt++;
			mask=mask>>1;
			i=i+2;
			if (shift==0xFF) {
				tmp=0;
				mask=0x7F;
				shift=7;
				data->nachricht[cnt_txt] = ((hex&mask) << (7-shift)) | tmp;
				tmp=hex>>shift;	
				cnt_txt++;
				shift--;
				mask=mask>>1;
			}
		}
		data->nachricht[cnt_txt]='\0';	// '0' anfgen , Endestring	
	}
}


/**
* Stellt eine Textnachricht auf dem Mobiltelefon-Display dar
* @param nachricht Pointer auf darzustellende Nachricht auf dem Mobiltelefon-Display
*/
void sms_display_text(char* nachricht)
{	
	char tmp[20]="";
	uint8_t i;
	uint8_t cnt_txt=0;

	UCSRB &= ~(1 << RXCIE);		// UART Receive ISR ausstellen
	uart_clear();

	cnt_txt = strlen(nachricht);
	printf("AT^SSTK=%02i,0\r\n",cnt_txt+13+1);	// 13 Header, 1 't' vor nachricht
	gets(tmp);
	printf("D0");	// Text String Tag
	printf("%02X",cnt_txt+13+1-2);	// Lnge des PDU String - 1
	/*
	01	--> Command Detail Tag
	03	--> Lnge
	01	--> Command Number
	21	--> Type of Command (Display Text)
	80	--> Command Qualifier (Display TExt, wait for user clear message)
	02	--> Device Identity Tag
	02	--> Lnge
	81	--> Source Device Identity (SIM)
	02	--> Destination Device Identity (Display)
	0D	--> Text String Tag
	*/
	printf("0103012180020281020D");	//Header
	printf("%02X",cnt_txt+1);	// Lnge des Text String
	printf("%02X",'t');			// 't' vor nachricht einfgen
	// Text String in Hex umwandeln
	for ( i=0 ; i<cnt_txt ; i++ ){
		printf("%02X",nachricht[i]);
	}
	printf("%c",26);	// ENDE Zeichen, strg+z

	_delay_ms(200);		// auf Antwort von Mobiltelefon warten
	uart_clear();		
	// UART RX ISR einschalten
	if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }
}


/**
* Sucht nach bereinstimmungen der bergebenen Telefonnummer in den gespeicherten Listen (\ref sms_telnr_list oder \ref SMS_TELNR)
* @param telnr_in Vergleichsnummer zum berprfen
* @return gibt an, ob eine entsprechende Nummer in \ref sms_telnr_list oder \ref SMS_TELNR gefunden wurde (0 - kein Fund, 1 - Fund)
*/
uint8_t sms_check_nr(char* telnr_in)
{
	// Anrufererkennung (aus SMS_TELNR oder SMS_TELNR_LIST_T)
	// Anrufer hat Telefonummer 0162*****
	// (fhrende "0" wird gelscht, da Mobiltelefon mit "+49162..."
	// oder mit "0162..." antworten kann --> nicht strcmp sondern strstr berwenden)
	uint8_t i;
	char sms_telnr_tmp1[SMS_TELNR_MAX_LEN+1];
	char sms_telnr_tmp2[SMS_TELNR_MAX_LEN+1]={SMS_TELNR};
	char * p_sms_telnr_tmp1;
	char * p_sms_telnr_tmp2;
	p_sms_telnr_tmp2 = sms_telnr_tmp2;
	p_sms_telnr_tmp2++;		// fhrende 0 lschen (pointer beginnt erst bei "1")
	for (i=0; i<SMS_TELNR_LIST_MAX+1; i++) {
		strcpy(sms_telnr_tmp1,sms_telnr_list[i].telnr);
		p_sms_telnr_tmp1 = sms_telnr_tmp1;
		if (*p_sms_telnr_tmp1=='0') {
			p_sms_telnr_tmp1++;		// fhrende 0 bzw + lschen (pointer beginnt erst bei "1" bzw "49")
		}
		if (*p_sms_telnr_tmp1=='+') {
			// fhrendes +49 lschen (pointer beginnt erst bei "1")
			p_sms_telnr_tmp1++;		
			p_sms_telnr_tmp1++;	
			p_sms_telnr_tmp1++;	
		}
		if ( strstr(telnr_in,p_sms_telnr_tmp1) || strstr(telnr_in,p_sms_telnr_tmp2)){
			return 1;
		}
	}
	return 0;
}


/**
* State Machine zum auswerten der eingegangenen Daten vom Mobiltelefon
* @return gibt an, ob ein entsprechender Eintrag in der State Machine gefunden wurde (entsprechend \ref SMS_MSG) 
*/
SMS_MSG sms_state_machine(void)
{
	char sms_tmp[UART_MAXSTRLEN+1]={'0'};

	if( uart_str_complete==1 ){

		if ( strstr((char*)uart_string,"OK") ){
			uart_str_complete=0;
			return SMS_OK;
		}

		if ( strstr((char*)uart_string,"ERROR") ){
			uart_str_complete=0;
			return SMS_ERROR;
		}
		// Anruf geht ein
		if ( strcmp("RING",(char*)uart_string)==0 ) {		//Klingen, auch "+CRING: VOICE" mgl.
			UCSRB &= ~(1 << RXCIE);		// uart RX ISR deaktivieren
			anz_klingeln++;
			if (anz_klingeln==1) {
				uart_clear();
				printf("AT+CLCC\r\n");				//Rufnummer abfragen
				uart_str_complete=0;
				if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }
				return SMS_REQUEST_TELNR;
			}
			if (anz_klingeln>=3) {
				uart_clear();
				printf("AT+CHUP\r\n");				//Auflegen
				anz_klingeln=0;
				uart_str_complete=0;
				if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }				
				return SMS_CANCEL_RING;
			}
			// UART RX ISR aktivieren
			uart_str_complete=0;	
			if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }
			return SMS_INCOMING_RING;
		}

		// Anrufererkennung (aus SMS_TELNR oder SMS_TELNR_LIST_T)
		// in uart_string steht Antwort von AT+CLCC
		if ( strstr((char*)uart_string,"+CLCC:") ){
			UCSRB &= ~(1 << RXCIE);		// uart RX ISR deaktivieren
			uart_clear();
			printf("AT+CHUP\r\n");	//Auflegen
			_delay_ms(100);	// auf Antwort von AT+CHUP warten
			anz_klingeln=0;
				
			//global bekannt geben, wer anruft
			int dummy=0;
			char tmp[SMS_TELNR_MAX_LEN+1];
			sscanf((char*)uart_string,"+CLCC: %i,%i,%i,%i,%i,\"%[^\"]\",%i",&dummy,&dummy,&dummy,&dummy,&dummy,tmp,&dummy);
			strcpy(sms_last_caller,tmp);

			if (sms_check_nr((char*)tmp)){
				uart_clear();
				// UART RX ISR aktivieren	
				if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }
				uart_str_complete=0;
				return SMS_RETRIEVE_TELNR;
			}
			uart_clear();
			// UART RX ISR aktivieren	
			if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }
			uart_str_complete=0;
			return SMS_RETRIEVE_nTELNR;
		}

		// neue SMS eingegangen
		if ( strstr((char*)uart_string,"+CMTI:") ){
			UCSRB &= ~(1 << RXCIE);		// uart RX ISR deaktivieren
			int sms_index=0;
			int dummy=0;

			sscanf((char*)uart_string,"+CMTI: \"ME\",%i",&sms_index);
			_delay_ms(200);	// auf Antwort von +CMTI warten
			uart_clear();

			// SMS auslesen
			printf("AT+CMGR=%i\r\n",sms_index);
			gets(sms_tmp);	// fhrendes \r\n verwerfen
			scanf("+CMGR: %i,,%i\r\n%s\r\n",&dummy,&dummy,sms_tmp);
			_delay_ms(400);	// auf Antwort von +CMGR warten, \r\nOK\r\n oder \r\nERROR\r\n
			uart_clear();	// Antwort von CMGR lschen

			//empfangende PDU Zeichenkette decodieren
			sms_decode(sms_tmp,&sms_dec_data);

			// SMS lschen
			printf("AT+CMGD=%i\r\n",sms_index);
			_delay_ms(100);	// auf Antwort von AT+CMGD warten

			if ( sms_check_nr((char*)sms_dec_data.telnr)){		
				// Kommando aus empfangener SMS auslesen
				char *ptr_nachricht;
				char nachricht[SMS_MESSAGE_MAX_LEN+1];	// strtok verndert string --> Kopie anlegen
				SMS_CMD_LIST_T keyItem = {"",NULL};
				strcpy(nachricht,(char*)sms_dec_data.nachricht);
				//nachricht in Kleinbuchstaben umwandeln
				ptr_nachricht = nachricht;	// Startadresse bergeben
				while(*ptr_nachricht){
					if (*ptr_nachricht >= 'A' && *ptr_nachricht <= 'Z') {
						*ptr_nachricht = *ptr_nachricht - 'A' + 'a';
					}
					ptr_nachricht++;
				}
				// sortieren der Befehlsliste (Voraussetzung fr bsearch)
				char *tmp;
				qsort(&sms_cmd_list, (sizeof(sms_cmd_list)/sizeof(SMS_CMD_LIST_T)), sizeof(SMS_CMD_LIST_T), (int(*)(const void*,const void*))strcmp);
				ptr_nachricht = strtok_r(nachricht,SMS_DELIMITER,&tmp);	// ersten Befehl extrahieren
				while (ptr_nachricht!=NULL){
					// Befehle entsprechend Liste ausfhren
					// nach www.wachtler.de/informatik_2/node58.html
					strcpy(keyItem.cmd,ptr_nachricht);	// gefundenes Kommando bergeben
		      		SMS_CMD_LIST_T* spItem = (SMS_CMD_LIST_T*)bsearch (&keyItem, sms_cmd_list, (sizeof(sms_cmd_list)/sizeof(SMS_CMD_LIST_T)), sizeof(SMS_CMD_LIST_T), (int(*)(const void*,const void*)) strcmp);
		      		if (spItem != NULL) { spItem->fp(0); }
					ptr_nachricht = strtok_r(NULL,SMS_DELIMITER,&tmp);	// neues Kommando suchen (falls mehrerer vorhanden sind)
				}
			}
			uart_clear();
			// UART RX ISR aktivieren	
			if (UART_ISR!=0) { UCSRB |= (1 << RXCIE); }
			uart_str_complete=0;
			return SMS_MSG_DECODE;
		}
		uart_str_complete=0;
	}
	return SMS_IDLE;
}



uint8_t sms_cmdset(uint8_t dat)
{
	PORTD ^= ( 1 << PD6 );
	return 0;
};



